#include "tinysvg.h"
#include <stdio.h>
#include <SDL/SDL.h>
#include <GL/GL.h>
#include <time.h>
#include "artgl.h"
#include "svgsprite.h"

#include <libart_lgpl/libart.h>


void do_fps(clock_t* begin, unsigned long* fc) {
  clock_t now = clock();
  double elapsed = (double)(now - *begin) / CLOCKS_PER_SEC;
  printf("%f FPS\n", *fc/elapsed);
  printf("%f msec per frame\n", 1000 * elapsed / *fc);
  *begin = now;
  *fc = 0;
} 

int main(int argc, char** argv) {
  int i;
  double tx[6];
  
  /* Initialize the SDL library */
  if( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr,
            "Couldn't initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }

  /* Clean up on exit */
  atexit(SDL_Quit);

  int flags = SDL_OPENGL;
  SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 0 );
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
  SDL_GL_SetAttribute( SDL_GL_MULTISAMPLEBUFFERS, 1);
  SDL_GL_SetAttribute( SDL_GL_MULTISAMPLESAMPLES, 4);

  SDL_Surface* screen = SDL_SetVideoMode(640, 480, 32, flags);
  
  if (screen == NULL) {
    fprintf(stderr, "Couldn't set video mode: %s\n", SDL_GetError());
    exit(-1);
  }
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, screen->w, screen->h, 0.0, -1, 1);
  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
    
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  SDL_WM_SetCaption("LD48", 0);

  art_affine_identity(tx);
  printf("tx is:\n");
  for (i=0; i<6; ++i) {
    printf("  tx[%d] = %lf\n", i, tx[i]);
  }

  art_affine_rotate(tx, 90);
  printf("tx is:\n");
  for (i=0; i<6; ++i) {
    printf("  tx[%d] = %lf\n", i, tx[i]);
  }
  
  art_affine_translate(tx, 2, 4);
  printf("tx is:\n");
  for (i=0; i<6; ++i) {
    printf("  tx[%d] = %lf\n", i, tx[i]);
  }

  printf("hello, world!\n");

  ArtLibrary* l = artlibrary_load();
  artlibrary_free(l);

  TinySvg* t = tinysvg_load("svg/test.svg");
  //art_affine_scale(t->root->transform, 2, 2);

  printf("svg info:\n");
  printf("  %d groups, %d paths, %d verts\n", t->n_groups, t->n_paths, t->n_verts);
  printf("  extents: (%.3lf, %.3lf) - (%.3lf, %.3lf)\n", t->minx, t->miny, t->maxx, t->maxy);
  printf("  w = %.3lf, h = %.3lf, tx = (%.3lf, %.3lf)\n", t->w, t->h, t->tx, t->ty);

  int done = 0;
  double rot = 0;

  /*
  double eqn[4] = { 0.2, 1, 0, -100 };
  
  glClipPlane(GL_CLIP_PLANE0, eqn);
  glEnable(GL_CLIP_PLANE0);
  */

  clock_t begin = clock();
  unsigned long fc = 0;
  int invert = 0;
  int flip = 0;

  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

  while (!done) {

    SDL_Event event;

    /* Poll events */
    while(SDL_PollEvent(&event)) {

      switch(event.type) {
        
      case SDL_KEYDOWN:
        /* handle keyboard stuff here */
        switch( event.key.keysym.sym ){
        case SDLK_ESCAPE:
          done = 1;
          break;
        case SDLK_a:
          rot -= 5;
          break;
        case SDLK_d:
          rot += 5;
          break;
        case SDLK_i:
          invert = !invert;
          break;
        case SDLK_f:
          flip = !flip;
          break;
        default:
          break;
        }
        break;

        
      case SDL_QUIT:
        done = 1;
        break;
        
      default:
        break;
      }
    }

    // re-draw screen
    glClear(GL_COLOR_BUFFER_BIT);

    glPushMatrix();
    glTranslated(screen->w*0.5, screen->h*0.5, 0);
    glRotated(rot, 0, 0, 1);
    glTranslated(-t->w*0.5, t->h*0.5, 0);
    tinysvg_draw(t, flip, invert);
    glPopMatrix();



    SDL_GL_SwapBuffers();
    
    if (++fc == 500) { do_fps(&begin, &fc); }

  }

  do_fps(&begin, &fc);
  
  tinysvg_free(t);

  return 0;

}
