#ifndef _TINYSVG_H_
#define _TINYSVG_H_

#include <libart_lgpl/libart.h>

typedef struct TINYSVGPATH TinySvgPath;
typedef struct TINYSVGGROUP TinySvgGroup;
typedef struct TINYSVGNODE TinySvgNode;
typedef struct TINYSVG TinySvg;

#define TINYSVG_TYPE_GROUP 0
#define TINYSVG_TYPE_PATH  1

struct TINYSVGNODE {
  int type;
  TinySvgGroup* parent;
};

struct TINYSVGPATH {
  int type;
  TinySvgGroup* parent;
  char id[32];
  unsigned int fill_rgba;
  unsigned int stroke_rgba;
  double stroke_width;
  unsigned int n_verts;
  ArtVpath* vpath;
};

struct TINYSVGGROUP {
  int type;
  TinySvgGroup* parent;
  char id[32];
  double transform[6];
  unsigned int n_children;
  void** children;
};

struct TINYSVG {
  char filename[64];
  unsigned int n_groups;
  unsigned int n_paths;
  unsigned int n_verts;
  double minx;
  double maxx;
  double miny;
  double maxy;
  double w;
  double h;
  double tx;
  double ty;
  TinySvgGroup* root;
};

TinySvg* tinysvg_load(const char* filename);

TinySvgGroup* tinysvg_group_create(TinySvg* t, const char* id);
TinySvgGroup* tinysvg_group_add(TinySvg* t, TinySvgGroup* g, const char* id);
TinySvgPath* tinysvg_path_add(TinySvg* t, TinySvgGroup* g, const char* id, ArtVpath* vp);

void tinysvg_free(TinySvg*);
void tinysvg_group_free(TinySvg* t, TinySvgGroup*);
void tinysvg_path_free(TinySvg* t, TinySvgPath*);

void tinysvg_draw(TinySvg*, int flip, int invert);

#endif
